@@ -27,6 +27,8 @@ extras["standard"] = combine(extras["matplotlib"], extras["viz"])
 extras["dill"] = ["dill"]
 extras["dask"] = ["dask[dataframe]"]
 extras["gcp"] = ["google-cloud-storage"]
+extras["parallel"] = ["cloudpickle", "loky"]
+extras["geopandas"] = ["geopandas"]
 
 extras["examples"] = combine(extras["standard"], ["scikit-learn"])
 extras["full"] = combine(*extras.values())
@@ -36,6 +38,8 @@ extras["dev"] = combine(
         "pytest",
         "black",
         "flake8",
+        "flake8-print",
+        "flake8-fixme",
         "sphinx",
         "sphinx_rtd_theme",
         "sphinx-autobuild",
