@@ -22,8 +22,8 @@ import numpy as np
 from pyarrow import parquet, Table
 import pandas as pd
 
+from .decoration import decorator_updating_accumulator
 from .exception import UnsupportedSerializedValueError
-from .provider import provider_wrapper, ProtocolUpdateProvider, is_func_or_provider
 from .optdep import import_optional_dependency
 from .util import (
     read_hashable_bytes_from_file_or_dir,
@@ -46,11 +46,16 @@ def check_is_like_protocol(obj):
             )
 
 
-class BaseProtocol(object):
+class BaseProtocol:
     def validate(self, value):
+        """
+        Checks if a value is valid for this protocol. Throws an exeception if the value
+        is invalid; otherwise does nothing.
+        """
         pass
 
     def value_is_valid(self, value):
+        """Indicates whether the passed value is valid for this protocol."""
         try:
             self.validate(value)
         except Exception:
@@ -58,23 +63,54 @@ class BaseProtocol(object):
         return True
 
     def get_fixed_file_extension(self):
+        """
+        Returns a file extension identifying this protocol. This value will be appended
+        to the name of any file written by the protocol, and may be used to determine
+        whether a file can be read by the protocol.
+
+        This string should be unique, not shared with any other protocol. By
+        convention, it doesn't include an initial period, but may include periods in
+        the middle.  (For example, `"csv"`, and `"csv.zip"` would both be sensible
+        file extensions.)
+
+        If a protocol uses different file extensions for different input values, it
+        should implement ``file_extension_for_value`` instead.
+        """
         raise NotImplementedError()
 
     def file_extension_for_value(self, value):
+        """
+        Like ``get_fixed_file_extension``, but may provide different extensions
+        depending on the value to be serialized.
+        """
         return self.get_fixed_file_extension()
 
     def can_read_file_extension(self, extension):
+        """Indicates whether a file extension can be read by this protocol."""
         return extension == self.get_fixed_file_extension()
 
     def supports_value(self, value):
+        """Indicates whether a value can be serialized by this protocol."""
         return any(isinstance(value, type_) for type_ in self.get_all_supported_types())
 
     def write(self, value, path):
+        """Serializes the object ``value`` to the pathlib path ``path``."""
         raise NotImplementedError()
 
     def read(self, path):
+        """Deserializes an object from the pathlib path ``path``, and returns it."""
         raise NotImplementedError()
 
+    def read_with_extension(self, path, extension):
+        """
+        Deserializes an object from the pathlib path ``path``, and returns it.
+
+        May use the file extension ``extension`` to determine what deserialization
+        method to use. This extension must be the same extension generated by the
+        protocol that originally serialized the value.
+        """
+        return self.read(path)
+
     SIMPLE_TYPES = {
         bool,
         str,
@@ -86,15 +122,22 @@ class BaseProtocol(object):
     }
 
     def tokenize(self, value):
+        """
+        Converts a valid value into a unique hash which can be used as a filename.
+        """
         if type(value) in self.SIMPLE_TYPES:
             return tokenization.tokenize(value)
         else:
             return tokenization.tokenize(value, self._write_to_bytes)
 
     def tokenize_file(self, path):
+        """Like ``tokenize``, but operates on a serialized value."""
         return tokenization.tokenize(path, read_hashable_bytes_from_file_or_dir)
 
     def _write_to_bytes(self, value):
+        """
+        Like ``write``, but returns an array of bytes rather than writing to a file.
+        """
         file_name = "temp_file"
         temp_dir = Path(tempfile.mkdtemp())
         try:
@@ -125,13 +168,13 @@ class BaseProtocol(object):
     #    def func(...):
     #        ...
     #
-    def __call__(self, func_or_provider=None, **kwargs):
-        if func_or_provider is not None:
+    def __call__(self, func=None, **kwargs):
+        if func is not None:
             if len(kwargs) > 0:
                 raise ValueError(
                     f"{self} can't be called with both a function and keywords"
                 )
-            if not is_func_or_provider(func_or_provider):
+            if not callable(func):
                 raise ValueError(
                     oneline(
                         f"""
@@ -142,8 +185,11 @@ class BaseProtocol(object):
                     )
                 )
 
-            wrapper = provider_wrapper(ProtocolUpdateProvider, self)
-            return wrapper(func_or_provider)
+            return decorator_updating_accumulator(
+                lambda acc: acc.update_attr(
+                    "protocol", self, "protocol_decorator", raise_if_already_set=False
+                )
+            )(func)
         else:
             return self.__class__(**kwargs)
 
@@ -155,16 +201,24 @@ class PicklableProtocol(BaseProtocol):
     """
     Decorator indicating that an entity's values can be serialized using the
     ``pickle`` library.
+
+    Parameters:
+        pickle_protocol_version: int (default: 4)
+            The pickle serialization protocol to use.
     """
 
+    def __init__(self, pickle_protocol_version=4):
+        super(PicklableProtocol, self).__init__()
+        self._pickle_protocol_version = pickle_protocol_version
+
     def get_fixed_file_extension(self):
         return "pkl"
 
     def write(self, value, path):
         with path.open("wb") as file_:
-            pickle.dump(value, file_)
+            pickle.dump(value, file_, protocol=self._pickle_protocol_version)
 
-    def read(self, path, extension):
+    def read(self, path):
         with path.open("rb") as file_:
             return pickle.load(file_)
 
@@ -203,7 +257,7 @@ class DillableProtocol(BaseProtocol):
         with path.open("wb") as file_:
             self._get_dill_module().dump(value, file_)
 
-    def read(self, path, extension):
+    def read(self, path):
         with path.open("rb") as file_:
             return self._get_dill_module().load(file_)
 
@@ -231,7 +285,7 @@ class ParquetDataFrameProtocol(BaseProtocol):
     def validate(self, value):
         assert isinstance(value, pd.DataFrame)
 
-    def read(self, path, extension):
+    def read(self, path):
         with path.open("rb") as file_:
             return parquet.read_table(file_).to_pandas()
 
@@ -294,7 +348,7 @@ class FeatherDataFrameProtocol(BaseProtocol):
     def validate(self, value):
         assert isinstance(value, pd.DataFrame)
 
-    def read(self, path, extension):
+    def read(self, path):
         with path.open("rb") as file_:
             return pd.read_feather(file_)
 
@@ -324,7 +378,7 @@ class ImageProtocol(BaseProtocol):
         assert Image is not None
         assert isinstance(value, Image.Image)
 
-    def read(self, path, extension):
+    def read(self, path):
         with path.open("rb") as file_:
             Image = import_optional_dependency(
                 "PIL.Image", purpose="the @image decorator"
@@ -354,7 +408,7 @@ class NumPyProtocol(BaseProtocol):
     def validate(self, value):
         assert isinstance(value, np.ndarray)
 
-    def read(self, path, extension):
+    def read(self, path):
         with path.open("rb") as file_:
             return np.load(file_)
 
@@ -384,7 +438,7 @@ class DaskProtocol(BaseProtocol):
         assert dd is not None
         assert isinstance(value, dd.DataFrame)
 
-    def read(self, path, extension):
+    def read(self, path):
         dd = import_optional_dependency("dask.dataframe", purpose="the @dask decorator")
         with warnings.catch_warnings():
             warnings.filterwarnings("error", message=r".*cannot\s+autodetect\s+index.*")
@@ -421,7 +475,7 @@ class YamlProtocol(BaseProtocol):
         with path.open("w") as file_:
             yaml.dump(value, file_, **self._kwargs)
 
-    def read(self, path, extension):
+    def read(self, path):
         with path.open("r") as file_:
             return yaml.safe_load(file_)
 
@@ -482,7 +536,7 @@ class PathProtocol(BaseProtocol):
         else:
             raise AssertionError(f"Unexpected operation: {self.operation!r}")
 
-    def read(self, path, extension):
+    def read(self, path):
         return single_element(path.iterdir())
 
 
@@ -537,13 +591,14 @@ class CombinedProtocol(BaseProtocol):
         else:
             self._subprotocols[-1].validate(value)
 
-    def read(self, path, extension):
-        if not self.can_read_file_extension(extension):
+    def read_with_extension(self, path, extension):
+        protocol = self._protocol_for_extension(extension)
+        if protocol is None:
             raise ValueError(
                 "This protocol doesn't know how to read a file with "
                 f"extension {extension!r}"
             )
-        return self._protocol_for_extension(extension).read(path, extension)
+        return protocol.read_with_extension(path, extension)
 
     def write(self, value, path):
         self._protocol_for_value(value).write(value, path)
@@ -604,3 +659,69 @@ class EnumProtocol(PicklableProtocol):
 
     def __repr__(self):
         return f"EnumProtocol{tuple(self._allowed_values)!r}"
+
+
+gpd = import_optional_dependency("geopandas", raise_on_missing=False)
+
+
+class GeoPandasProtocol(BaseProtocol):
+    """
+    Decorator indicating that an entity's values always have the
+    ``geopandas.geodataframe.GeoDataFrame`` type.
+
+    These values will be serialized to SHP files.
+    """
+
+    def get_fixed_file_extension(self):
+        return "shp"
+
+    def write(self, value, path):
+        self._check_column_name_length_restriction(value)
+        value.to_file(path)
+
+    def _check_column_name_length_restriction(self, df):
+        long_columns = [c for c in df.columns if len(c) > 10]
+        if long_columns:
+            raise ValueError(
+                oneline(
+                    f"""
+                    GeoDataFrames truncate all column names to length 10 due
+                    to constraints from ShapeFiles. Column names {long_columns}
+                    have length greater than 10. You can fix this by renaming
+                    your columns to be a shorter length."""
+                )
+            )
+
+    def read(self, path):
+        gpd = import_optional_dependency("geopandas", purpose="deserialize a .shp file")
+        return gpd.read_file(path)
+
+    def validate(self, value):
+        assert gpd is not None
+        assert isinstance(value, gpd.GeoDataFrame)
+
+
+class TupleProtocol(BaseProtocol):
+    """
+    Describes values that are Python tuples of a fixed length. This is used mainly
+    by Bionic's infrastructure for values corresponding to tuple descriptors.
+
+    This protocol does not support serializing or deserializing values.
+    """
+
+    def __init__(self, length):
+        super(TupleProtocol, self).__init__()
+
+        self._expected_length = length
+
+    def validate(self, value):
+        try:
+            items = tuple(value)
+        except TypeError as e:
+            raise AssertionError(str(e))
+        if len(items) != self._expected_length:
+            message = f"""
+            Expected a sequence with length {self._expected_length};
+            instead, got {len(items)} values: {items!r}
+            """
+            raise AssertionError(oneline(message))
