@@ -2,17 +2,21 @@
 Miscellaneous utility functions.
 """
 
-import logging
 from collections import defaultdict
 from hashlib import sha256
 from binascii import hexlify
 import re
 import subprocess
+import threading
 import warnings
 import shutil
 
 from .optdep import import_optional_dependency, oneline
 
+import logging
+
+logger = logging.getLogger(__name__)
+
 
 def n_present(*items):
     "Returns the number of non-None arguments."
@@ -74,7 +78,14 @@ def single_element(iterable):
         )
     return items[0]
 
-    return next(iter(iterable))
+
+def single_unique_element(iterable):
+    """
+    Takes an iterable with exactly one unique element and returns that element.
+
+    Equivalent to `single_element(set(iterable))`.
+    """
+    return single_element(set(iterable))
 
 
 def hash_to_hex(bytestring, n_bytes=None):
@@ -123,6 +134,7 @@ def get_gcs_client_without_warnings(cache_value=True):
         warnings.filterwarnings(
             "ignore", "Your application has authenticated using end user credentials"
         )
+        logger.info("Initializing GCS client ...")
         return gcs.Client()
 
 
@@ -190,7 +202,11 @@ def read_hashable_bytes_from_file_or_dir(path):
 
 
 def ensure_parent_dir_exists(path):
-    path.parent.mkdir(parents=True, exist_ok=True)
+    ensure_dir_exists(path.parent)
+
+
+def ensure_dir_exists(path):
+    path.mkdir(parents=True, exist_ok=True)
 
 
 def recursively_copy_path(src_path, dst_path):
@@ -255,7 +271,7 @@ def rewrap_docstring(docstring):
     return "\n".join(paragraphs)
 
 
-class ImmutableSequence(object):
+class ImmutableSequence:
     def __init__(self, items):
         self.__items = tuple(items)
 
@@ -352,55 +368,7 @@ class ImmutableMapping(ImmutableSequence):
         return f"{self.__class__.__name__}({self.__values_by_key!r})"
 
 
-# TODO I'm not sure if we'll end up needing this or not.
-class ExtensibleLogger(object):
-    def __init__(self, logger):
-        self._logger = logger
-
-    # Subclasses should call this.
-    def base_log(self, level, msg, *args, **kwargs):
-        self._logger.log(level, msg, *args, **kwargs)
-
-    # Subclasses should override this.
-    def custom_log(self, level, msg, *args, **kwargs):
-        self.base_log(level, msg, *args, **kwargs)
-
-    # Internal methods.
-    def _custom_log_if_enabled(self, level, msg, *args, **kwargs):
-        if not self.isEnabledFor(level):
-            return
-        self.custom_log(level, msg, *args, **kwargs)
-
-    # User-facing methods.
-    def isEnabledFor(self, level):
-        return self._logger.isEnabledFor(level)
-
-    def log(self, level, msg, *args, **kwargs):
-        self._custom_log_if_enabled(level, msg, *args, **kwargs)
-
-    def debug(self, msg, *args, **kwargs):
-        self._custom_log_if_enabled(logging.DEBUG, msg, *args, **kwargs)
-
-    def info(self, msg, *args, **kwargs):
-        self._custom_log_if_enabled(logging.INFO, msg, *args, **kwargs)
-
-    def warning(self, msg, *args, **kwargs):
-        self._custom_log_if_enabled(logging.WARNING, msg, *args, **kwargs)
-
-    def warn(self, msg, *args, **kwargs):
-        self.warning(msg, *args, **kwargs)
-
-    def error(self, msg, *args, **kwargs):
-        self._custom_log_if_enabled(logging.ERROR, msg, *args, **kwargs)
-
-    def critical(self, msg, *args, **kwargs):
-        self._custom_log_if_enabled(logging.CRITICAL, msg, *args, **kwargs)
-
-    def exception(self, msg, *args, **kwargs):
-        self._custom_log_if_enabled(logging.ERROR, msg, *args, exc_info=True, **kwargs)
-
-
-class FileCopier(object):
+class FileCopier:
     """
     A wrapper for a Path object, exposing a ``copy`` method that will copy
     the underlying file to a local or cloud destination.
@@ -446,3 +414,29 @@ def init_basic_logging(level=logging.INFO):
         format="%(asctime)s %(levelname)s %(name)16s: %(message)s",
         datefmt="%Y-%m-%d %H:%M:%S",
     )
+
+
+class SynchronizedSet:
+    """
+    A set-like class that only exposes add() and contains() method.
+    The add() operation returns whether a new value was added and is
+    atomic.
+    """
+
+    def __init__(self):
+        self.values = set()
+        self.lock = threading.Lock()
+
+    def add(self, value):
+        """
+        Adds the value and returns True if the value isn't present in the
+        set. Returns False if the set already contains the value.
+        """
+        with self.lock:
+            if self.contains(value):
+                return False
+            self.values.add(value)
+            return True
+
+    def contains(self, value):
+        return value in self.values
