@@ -5,8 +5,10 @@ import io
 import pickle
 from pathlib import Path
 import contextlib
+import threading
 
 import pandas as pd
+import pandas.testing as pdt
 
 import bionic as bn
 from bionic.exception import (
@@ -15,9 +17,10 @@ from bionic.exception import (
     EntitySerializationError,
     IncompatibleEntityError,
     UndefinedEntityError,
+    UnsetEntityError,
 )
 
-from ..helpers import count_calls, assert_re_matches
+from ..helpers import assert_re_matches, count_calls
 
 
 @pytest.fixture(scope="function")
@@ -162,7 +165,7 @@ def test_add_case(preset_builder):
     builder.add_case("z", 7)
     assert builder.build().get("z", set) == {2, 3, 7}
 
-    with raises(ValueError):
+    with raises(IncompatibleEntityError):
         builder.add_case("f", 7)
 
     with raises(UndefinedEntityError):
@@ -173,13 +176,17 @@ def test_add_case(preset_builder):
     assert builder.build().get("p", set) == {4, 5}
     assert builder.build().get("q", set) == {5, 6}
 
-    with raises(ValueError):
+    with raises(IncompatibleEntityError):
         builder.add_case("p", 7)
-    with raises(ValueError):
+    with raises(IncompatibleEntityError):
         builder.add_case("p", 4, "q", 6)
     builder.declare("r")
-    with raises(ValueError):
+    with raises(IncompatibleEntityError):
         builder.add_case("p", 1, "q", 2, "r", 3)
+    with raises(IncompatibleEntityError):
+        builder.add_case("p", 1, "r", 3)
+    with raises(IncompatibleEntityError):
+        builder.add_case("x", 1, "y", 2)
 
     with raises(IncompatibleEntityError):
         builder.add_case("y_plus", 2)
@@ -187,6 +194,21 @@ def test_add_case(preset_builder):
         builder.add_case("y_plus", 2, "y_plus_plus", 3)
 
 
+def test_add_case_out_of_order(builder):
+    builder.declare("p")
+    builder.declare("q")
+    builder.declare("r")
+
+    builder.add_case("p", 1, "q", 10, "r", 100)
+    builder.add_case("p", 2, "r", 200, "q", 20)
+    builder.add_case("r", 300, "q", 30, "p", 3)
+
+    flow = builder.build()
+    flow.get("p", set) == {1, 2, 3}
+    flow.get("q", set) == {10, 20, 30}
+    flow.get("r", set) == {100, 200, 300}
+
+
 def test_then_set(preset_builder):
     builder = preset_builder
 
@@ -299,7 +321,7 @@ def test_merge(builder):
     builder3.declare("b")
 
     @builder3  # noqa: F811
-    def h(a, b):
+    def h(a, b):  # noqa: F811
         return a * b
 
     builder.merge(builder3.build(), keep="new")
@@ -315,15 +337,17 @@ def test_merge(builder):
 def test_get_single(preset_flow):
     flow = preset_flow
 
-    with raises(ValueError):
+    with raises(UnsetEntityError):
         flow.get("x")
 
     assert flow.get("y") == 1
 
     with raises(ValueError):
         assert flow.get("z")
-    with raises(ValueError):
+
+    with raises(UnsetEntityError) as excinfo:
         assert flow.get("f")
+    assert "'x'" in str(excinfo.value)
 
     assert flow.get("p") == 4
     assert flow.get("q") == 5
@@ -347,6 +371,7 @@ def test_get_multiple(preset_flow):
     assert flow.get("q", set) == {5}
 
 
+@pytest.mark.run_with_all_execution_modes_by_default
 def test_get_collections(preset_flow):
     flow = preset_flow
 
@@ -389,35 +414,35 @@ def test_get_collections(preset_flow):
         assert list(p_series_index_df["q"]) == [5]
 
 
-def test_get_modes_persisted(preset_flow, tmp_path):
+@pytest.mark.parametrize("name", ["y", "y_fxn"])
+def test_get_modes_persisted(preset_flow, name, tmp_path):
     flow = preset_flow
-    name = "y_fxn"
 
     for mode in [object, "object"]:
         assert flow.get(name, mode=mode) == 1
 
     for mode in [Path, "path"]:
-        y_fxn_path = flow.get(name, mode=mode)
-        assert pickle.loads(y_fxn_path.read_bytes()) == 1
+        path = flow.get(name, mode=mode)
+        assert pickle.loads(path.read_bytes()) == 1
 
     flow.get(name, mode="FileCopier").copy(destination=tmp_path)
     serialized_fname = name + ".pkl"
     expected_file_path = tmp_path / serialized_fname
     assert pickle.loads(expected_file_path.read_bytes()) == 1
 
-    y_fxn_filename = flow.get(name, mode="filename")
-    assert isinstance(y_fxn_filename, str)
-    assert y_fxn_filename == str(y_fxn_path)
+    filename = flow.get(name, mode="filename")
+    assert isinstance(filename, str)
+    assert filename == str(path)
 
 
-@pytest.mark.parametrize("name", ["y", "y_fxn_no_persist"])
-def test_get_modes_not_persisted(preset_builder, name):
+def test_get_modes_not_persisted(preset_builder):
     @preset_builder
     @bn.persist(False)
     def y_fxn_no_persist(y):
         return y
 
     flow = preset_builder.build()
+    name = "y_fxn_no_persist"
 
     for mode in [object, "object"]:
         assert flow.get(name, mode=mode) == 1
@@ -572,39 +597,41 @@ def test_all_entity_names(preset_flow):
     }
 
 
-def test_in_memory_caching(builder):
+def test_in_memory_caching(builder, make_counter):
     builder.assign("x", 2)
     builder.assign("y", 3)
 
+    counter = make_counter()
+
     @builder
     @bn.persist(False)
-    @count_calls
+    @count_calls(counter)
     def xy(x, y):
         return x * y
 
     flow = builder.build()
 
     assert flow.get("xy") == 6
-    assert xy.times_called() == 1
+    assert counter.times_called() == 1
 
     assert flow.get("xy") == 6
-    assert xy.times_called() == 0
+    assert counter.times_called() == 0
 
     flow = builder.build()
 
     assert flow.get("xy") == 6
-    assert xy.times_called() == 1
+    assert counter.times_called() == 1
 
     new_flow = flow.setting("y", values=[4, 5])
 
     assert new_flow.get("xy", set) == {8, 10}
-    assert xy.times_called() == 2
+    assert counter.times_called() == 2
 
     assert new_flow.get("xy", set) == {8, 10}
-    assert xy.times_called() == 0
+    assert counter.times_called() == 0
 
     assert flow.get("xy") == 6
-    assert xy.times_called() == 0
+    assert counter.times_called() == 0
 
 
 def test_to_builder(builder):
@@ -643,7 +670,23 @@ def test_unhashable_index_values(builder):
     assert index_items == [[1, 2], [2, 3]]
 
 
-def test_entity_serialization_exception(builder):
+def test_unpicklable_non_persisted_entity(builder):
+    @builder
+    @bn.persist(False)
+    def unpicklable_lock():
+        return threading.Lock()
+
+    @builder
+    def uses_lock(unpicklable_lock):
+        unpicklable_lock.acquire()
+        unpicklable_lock.release()
+        return True
+
+    assert builder.build().get("uses_lock")
+
+
+@pytest.mark.run_with_all_execution_modes_by_default
+def test_entity_serialization_exception(builder, parallel_execution_enabled):
     @builder
     def unpicklable_value():
         def f():
@@ -655,10 +698,14 @@ def test_entity_serialization_exception(builder):
         builder.build().get("unpicklable_value")
     except EntitySerializationError as e:
         # AttributeError is what happens when we try to pickle a function.
-        assert isinstance(e.__cause__, AttributeError)
+        if parallel_execution_enabled:
+            assert "\nAttributeError:" in e.__cause__.tb
+        else:
+            assert isinstance(e.__cause__, AttributeError)
 
 
-def test_entity_computation_exception(builder):
+@pytest.mark.run_with_all_execution_modes_by_default
+def test_entity_computation_exception(builder, parallel_execution_enabled):
     @builder
     def uncomputable_value():
         return 1 / 0
@@ -666,7 +713,10 @@ def test_entity_computation_exception(builder):
     try:
         builder.build().get("uncomputable_value")
     except EntityComputationError as e:
-        assert isinstance(e.__cause__, ZeroDivisionError)
+        if parallel_execution_enabled:
+            assert "\nZeroDivisionError:" in e.__cause__.tb
+        else:
+            assert isinstance(e.__cause__, ZeroDivisionError)
 
 
 def test_multiple_compute_attempts(builder):
@@ -683,6 +733,17 @@ def test_multiple_compute_attempts(builder):
         flow.get("uncomputable_value")
 
 
+# Checks that a dataframe can be used as a fixed value. (This was added as a regression
+# test after finding a bug where Bionic was applying == to a fixed value, which doesn't
+# work for DataFrames.)
+def test_fixed_dataframe(builder):
+    df = pd.DataFrame(columns=["a", "b"], data=[[1, 2], [3, 4]],)
+
+    builder.assign("df", df)
+
+    pdt.assert_frame_equal(builder.build().get("df"), df)
+
+
 def test_entity_doc(builder):
     builder.declare("x")
     builder.declare("y", doc="y doc")
