@@ -10,7 +10,7 @@ from bionic.exception import (
 ALL_KEEP_VALUES = ["error", "old", "new", "combine"]
 
 
-class MergeTester(object):
+class MergeTester:
     """
     A utility class for testing many variations of flow merging.  This class
     is configured by creating several flows, associated them with names using
@@ -136,7 +136,7 @@ def merge_tester(builder):
 
     @fb  # noqa: F811
     @bn.outputs("x", "y")
-    def x(x_y):
+    def x(x_y):  # noqa: F811
         return x_y
 
     tester.add("DerivedJoint", fb.build())
@@ -165,7 +165,7 @@ def merge_tester(builder):
 
     @fb  # noqa: F811
     @bn.persist(False)
-    def x(root_x):
+    def x(root_x):  # noqa: F811
         return root_x ** 2
 
     tester.add("DS", fb.build())
@@ -176,7 +176,7 @@ def merge_tester(builder):
     @fb  # noqa: F811
     @bn.outputs("x", "y")
     @bn.persist(False)
-    def x(x_y):
+    def x(x_y):  # noqa: F811
         return x_y
 
     tester.add("DJ", fb.build())
